#!/usr/bin/python
# -*- coding: utf-8 -*-

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2017 Simon Effenberg <savar@schuldeigen.de>
# Copyright (c) 2017 Thumbor Community Extensions

from prometheus_client import Counter, start_http_server, Summary
from thumbor.metrics import BaseMetrics


class Metrics(BaseMetrics):

    def __init__(self, config):
        super().__init__(config)

        if not hasattr(Metrics, 'http_server_started'):
            port = config.PROMETHEUS_SCRAPE_PORT
            if isinstance(port, str):
                port = int(port)
            start_http_server(port)
            Metrics.http_server_started = True
            Metrics.counters = {}
            Metrics.summaries = {}

        # hard coded mapping right now
        self.mapping = {
                'response.status': ['statuscode'],
                'response.format': ['extension'],
                'response.bytes': ['extension'],
                'original_image.status': ['statuscode'],
                'original_image.fetch': ['statuscode', 'networklocation'],
                'response.time': ['statuscode_extension'],
        }

    def incr(self, metricname, value=1):
        name, labels = self.__data(metricname)

        if name not in Metrics.counters:
            Metrics.counters[name] = Counter(name, name, labels.keys())

        counter = Metrics.counters[name]

        if len(labels) != 0:
            counter = counter.labels(**labels)

        counter.inc(value)

    def timing(self, metricname, value):
        name, labels = self.__data(metricname)

        if name not in Metrics.summaries:
            Metrics.summaries[name] = Summary(name, name, labels.keys())

        summary = Metrics.summaries[name]

        if len(labels) != 0:
            summary = summary.labels(**labels)

        summary.observe(value)

    def __data(self, metricname):
        basename = self.__basename(metricname)

        return (self.__format(basename), self.__labels(basename, metricname))

    def __format(self, basename):
        # stolen from https://github.com/prometheus/statsd_exporter
        # _ -> __
        # - -> __
        # . -> _

        # following prometheus advice to prefix names with the app name
        return "thumbor_{0}".format(
            basename.replace('_','__').replace('-','__').replace('.','_')
        )

    def __labels(self, name, metricname):
        if name not in self.mapping:
            return {}

        # the split('.', MAXSPLIT) is mainly necessary to get the correct
        # stuff for original_image.fetch where the networklocation is
        # something like 'domain' so like 'test.com' and would be splitted at
        # least 1 time too often
        values = metricname.replace(name + '.', '').split('.', len(self.mapping[name])-1)
        labels = {}
        for index, label in enumerate(self.mapping[name]):
            labels[label] = values[index]

        return labels

    def __basename(self, metricname):
        for mapped in self.mapping.keys():
            if metricname.startswith(mapped + "."):
                metricname = mapped
        return metricname



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2022 Thumbor Community Extensions

from unittest import TestCase
from preggy import expect

from thumbor.context import Context
from thumbor.importer import Importer
from thumbor.config import Config
import thumbor.metrics
import tc_prometheus.metrics.prometheus_metrics

from tornado.httpclient import HTTPClient

class MetricsContext(TestCase):
    def setUp(self):
        self.config = Config()
        self.config.METRICS = 'tc_prometheus.metrics.prometheus_metrics'
        self.config.PROMETHEUS_SCRAPE_PORT = '8001'
        
        self.importer = Importer(self.config)
        self.importer.import_modules()
        
        self.context = Context(None, self.config, self.importer)

    def tearDown(self):
        pass


class CanCreateContextWithPrometheusMetrics(MetricsContext):
    def test_should_be_instance_of_prometheus_metrics(self):
        expect(self.context.metrics).to_be_instance_of(tc_prometheus.metrics.prometheus_metrics.Metrics)

    def test_should_not_fail_on_use(self):
        expect(self.context.metrics.incr('test.count')).not_to_be_an_error()
        expect(self.context.metrics.incr('test.count', 2)).not_to_be_an_error()
        expect(self.context.metrics.timing('test.time', 100)).not_to_be_an_error()


class PrometheusEndpoint(MetricsContext):
    def setUp(self):
        super().setUp()
        self.http_client = HTTPClient()
        
    def tearDown(self):
        super().tearDown()
        self.http_client.close()
    
    def test_should_present_metrics(self):
        self.context.metrics.incr('test.counter')
        self.context.metrics.incr('test.counter', 5)
        self.context.metrics.timing('test.timer', 150)
        self.context.metrics.timing('test.timer', 350)
        self.context.metrics.incr('response.status.200', 1)

        response = self.http_client.fetch('http://localhost:8001')

        expect(response.body).Not.to_be_null()

        body = str(response.body)

        expect(body).to_include('thumbor_test_counter_total 6')
        expect(body).to_include('thumbor_test_timer_count 2')
        expect(body).to_include('thumbor_test_timer_sum 500')
        expect(body).to_include('thumbor_response_status_total{statuscode="200"} 1')



# coding: utf-8

from setuptools import setup, find_packages

INSTALL_REQUIRES = [
    'thumbor==7.*,>=7.0.6',
    'prometheus_client==0.*,>=0.14.1',
]

setup(
    name='tc_prometheus',
    version="1.0.0",
    description='Thumbor Prometheus metrics extension',
    author='Simon Effenberg',
    author_email='savar@schuldeigen.de',
    url='https://github.com/thumbor-community/prometheus',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES,
    extras_require={
        "all": INSTALL_REQUIRES,
        "tests": INSTALL_REQUIRES + [
            'preggy==1.*,>=1.4.4',
            'pytest==7.*,>=7.0.0',
            'pytest-cov==3.*,>=3.0.0',
        ],
    },
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
    long_description="""
Prometheus metrics extension enables thumbor to expose a scrape endpoint prometheus can scrape from.
"""
)
